#ifndef POKEPLATINUM_UNK_02025E68_H
#define POKEPLATINUM_UNK_02025E68_H

#include "strbuf.h"
#include "struct_decls/struct_02025E6C_decl.h"

int sub_02025E68(void);
UnkStruct_02025E6C * sub_02025E6C(u32 param0);
void sub_02025E80(const UnkStruct_02025E6C * param0, UnkStruct_02025E6C * param1);
void sub_02025E8C(UnkStruct_02025E6C * param0);
BOOL sub_02025EA8(const UnkStruct_02025E6C * param0);
void sub_02025EC0(UnkStruct_02025E6C * param0, const u16 * param1);
void sub_02025EE0(UnkStruct_02025E6C * param0, const Strbuf *param1);
const u16 * sub_02025EF0(const UnkStruct_02025E6C * param0);
void sub_02025EF4(const UnkStruct_02025E6C * param0, Strbuf *param1);
Strbuf* sub_02025F04(const UnkStruct_02025E6C * param0, int param1);
void sub_02025F1C(UnkStruct_02025E6C * param0, u32 param1);
u32 sub_02025F20(const UnkStruct_02025E6C * param0);
u16 sub_02025F24(const UnkStruct_02025E6C * param0);
void sub_02025F2C(UnkStruct_02025E6C * param0, int param1);
u32 sub_02025F30(const UnkStruct_02025E6C * param0);
BOOL sub_02025F34(const UnkStruct_02025E6C * param0, int param1);
void sub_02025F48(UnkStruct_02025E6C * param0, int param1);
int sub_02025F58(const UnkStruct_02025E6C * param0);
u32 sub_02025F74(const UnkStruct_02025E6C * param0);
u32 sub_02025F78(UnkStruct_02025E6C * param0, u32 param1);
u8 sub_02025F8C(const UnkStruct_02025E6C * param0);
void sub_02025F90(UnkStruct_02025E6C * param0, u8 param1);
u32 sub_02025F94(UnkStruct_02025E6C * param0, u32 param1);
u32 sub_02025FB8(UnkStruct_02025E6C * param0, u32 param1);
u8 sub_02025FCC(const UnkStruct_02025E6C * param0);
void sub_02025FD0(UnkStruct_02025E6C * param0, u8 param1);
u8 sub_02025FD4(void);
u8 sub_02025FD8(const UnkStruct_02025E6C * param0);
void sub_02025FDC(UnkStruct_02025E6C * param0, u8 param1);
void sub_02025FE0(UnkStruct_02025E6C * param0);
int sub_02025FF0(UnkStruct_02025E6C * param0);
void sub_02025FF8(UnkStruct_02025E6C * param0);
int sub_02026004(UnkStruct_02025E6C * param0);
BOOL sub_0202600C(const UnkStruct_02025E6C * param0, const UnkStruct_02025E6C * param1);

#endif // POKEPLATINUM_UNK_02025E68_H
